@@ -880,23 +880,8 @@ editor_response_cb (GtkDialog *editor,
       
       if (err)
         {
-          GtkWidget *dialog;
-          
-          dialog = gtk_message_dialog_new (GTK_WINDOW (editor),
-                                           GTK_DIALOG_DESTROY_WITH_PARENT,
-                                           GTK_MESSAGE_ERROR,
-                                           GTK_BUTTONS_CLOSE,
-                                           _("There was an error displaying help: %s"),
-                                           err->message);
-          
-          g_signal_connect (G_OBJECT (dialog), "response",
-                            G_CALLBACK (gtk_widget_destroy),
-                            NULL);
-          
-          gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-          
-          gtk_widget_show (dialog);
-          
+          terminal_util_show_error_dialog (GTK_WINDOW (editor), NULL,
+                                           _("There was an error displaying help: %s"), err->message);
           g_error_free (err);
         }
     }
@@ -1286,6 +1271,8 @@ terminal_profile_edit (TerminalProfile *profile,
       g_signal_connect (G_OBJECT (w), "clicked",
 			G_CALLBACK (reset_compat_defaults_clicked),
 			profile);
+
+      terminal_util_set_unique_role (GTK_WINDOW (editor), "gnome-terminal-profile-editor");
     }
   else
     {
@@ -1512,7 +1499,7 @@ profile_editor_update_visible_name (GtkWidget       *editor,
   char *s;
   GtkWidget *w;
   
-  s = g_strdup_printf (_("Editing profile \"%s\""),
+  s = g_strdup_printf (_("Editing Profile \"%s\""),
                        terminal_profile_get_visible_name (profile));
   
   gtk_window_set_title (GTK_WINDOW (editor), s);
